Package: reticulate
Type: Package
Title: Interface to 'Python'
Version: 1.8
Authors@R: c(
  person("JJ", "Allaire", role = c("aut", "cre"), email = "jj@rstudio.com"),
  person("Kevin", "Ushey", role = c("aut"),
         email = "kevin@rstudio.com"),
  person("RStudio", role = c("cph", "fnd")),
  person("Yuan", "Tang", role = c("aut", "cph"),
         email = "terrytangyuan@gmail.com",
         comment = c(ORCID = "0000-0001-5243-233X")),
  person("Dirk", "Eddelbuettel", role = c("ctb", "cph"),
         email = "edd@debian.org"),
  person("Bryan", "Lewis", role = c("ctb", "cph"),
         email = "blewis@illposed.net"),
  person("Marcus", "Geelnard", role = c("ctb", "cph"),
         comment = "TinyThread library, http://tinythreadpp.bitsnbites.eu/")
  )
Description: Interface to 'Python' modules, classes, and functions. When calling
    into 'Python', R data types are automatically converted to their equivalent 'Python'
    types. When values are returned from 'Python' to R they are converted back to R
    types. Compatible with all versions of 'Python' >= 2.7.
License: Apache License 2.0
URL: https://github.com/rstudio/reticulate
BugReports: https://github.com/rstudio/reticulate/issues
SystemRequirements: Python (>= 2.7.0)
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.0)
Imports: utils, graphics, jsonlite, Rcpp (>= 0.12.7), Matrix
Suggests: testthat, knitr, callr, rmarkdown
LinkingTo: Rcpp
RoxygenNote: 6.0.1
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2018-06-08 11:45:19 UTC; jjallaire
Author: JJ Allaire [aut, cre],
  Kevin Ushey [aut],
  RStudio [cph, fnd],
  Yuan Tang [aut, cph] (<https://orcid.org/0000-0001-5243-233X>),
  Dirk Eddelbuettel [ctb, cph],
  Bryan Lewis [ctb, cph],
  Marcus Geelnard [ctb, cph] (TinyThread library,
    http://tinythreadpp.bitsnbites.eu/)
Maintainer: JJ Allaire <jj@rstudio.com>
Repository: CRAN
Date/Publication: 2018-06-08 13:23:00 UTC
Built: R 3.4.1; x86_64-w64-mingw32; 2018-08-03 19:37:58 UTC; windows
Archs: x64
