---
title: "Pandas Example"
output:
  html_document:
    df_print: paged
  pdf_document: default
---

```{r}
library(reticulate)
python <- "~/.virtualenvs/python-3.7.7-venv/bin/python"
if (file.exists(python))
  use_python(python, required = TRUE)
```

```{python}
import numpy as np
import pandas as pd

df = pd.DataFrame(np.random.rand(5, 2), columns=["x", "y"])

df.plot.scatter(x="x", y="y")
df.plot.scatter(x="x", y="y")
```
